import sys, pygame, world

class MiniMap:
    def __init__(self,screen, world, viewport):
        self.screen=screen
        self.world=world
        self.viewport=viewport

    def prerenderMap(self):
        self.cachedMap=pygame.Surface((40,40))

        width=self.world.getWidth()
        height=self.world.getHeight()
        for i in range(width):
            for j in range(height):
                tile=self.world.getTile(i,j)
                if tile==1:
                    color=(0,255,0) # flat terrain
                elif tile==2:
                    color=(128,128,128) # rock
                elif tile==3:
                    color=(50,50,255) # water
                self.cachedMap.set_at((40*i/width,40*j/height),color)

    def drawMap(self):
        map=pygame.Surface((self.cachedMap.get_width(),self.cachedMap.get_height()))
        map.blit(self.cachedMap,self.cachedMap.get_rect())
        
        # draw units
        for i in self.world.units:
            posx, posy=i.getPosition()
            posx=int(40*(1.0*posx/(16*self.world.getWidth())))
            posy=int(40*(1.0*posy/(16*self.world.getHeight())))
            # TODO: paint unit in team colors
            map.set_at((posx,posy),(0,0,0))
        
        # draw frame of viewport's visible portion
        width, height=self.viewport.getSize()
        posx, posy=self.viewport.getWorldPosition()
        width=int(40*(1.0*width/(16*self.world.getWidth())))
        height=int(40*(1.0*height/(16*self.world.getWidth())))
        posx=int(40*(-1.0*posx/(16*self.world.getWidth())))
        posy=int(40*(-1.0*posy/(16*self.world.getWidth())))

        # top left corner
        map.set_at((posx,posy),(255,255,255))
        map.set_at((posx+1,posy),(255,255,255))
        map.set_at((posx,posy+1),(255,255,255))
        # top right corner 
        map.set_at((posx+width,posy),(255,255,255))
        map.set_at((posx+width-1,posy),(255,255,255))
        map.set_at((posx+width,posy+1),(255,255,255))
        # bottom right corner 
        map.set_at((posx+width,posy+height),(255,255,255))
        map.set_at((posx+width-1,posy+height),(255,255,255))
        map.set_at((posx+width,posy+height-1),(255,255,255))
        # bottom left corner 
        map.set_at((posx,posy+height),(255,255,255))
        map.set_at((posx+1,posy+height),(255,255,255))
        map.set_at((posx,posy+height-1),(255,255,255))
       
        self.screen.blit(map,map.get_rect().move((20,500)))

if __name__=="__main__":
    import world
    pygame.init()
    screen=pygame.display.set_mode((800,600))
    world=world.World()
    world.loadMap("Test")
    miniMap=MiniMap(screen,world)
    miniMap.prerenderMap()
    while 1:
        for event in pygame.event.get():
            if event.type==pygame.QUIT:
                sys.exit()
            elif event.type==pygame.KEYDOWN:
                sys.exit()
        
        miniMap.drawMap()
        pygame.display.flip()
