#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <unistd.h>
#include <fcntl.h>

#include <iostream>

#include "connection.h"

Connection::Connection(int socketID)
{
  this->socketID=socketID;

  fcntl(socketID,F_SETFL,O_NONBLOCK);
}

Connection::~Connection()
{
  close(socketID);
}

void Connection::sendMessage(Message *m)
{
  char *buffer;
  
  buffer=new char[m->length+sizeof(unsigned short)+sizeof(int)];
  
  *(unsigned short *)buffer=m->length;
  *(int *)(buffer+sizeof(unsigned short))=m->messageID;
  memcpy(buffer+sizeof(unsigned short)+sizeof(int),m->buffer,m->length);

  send(socketID,buffer,m->length+sizeof(unsigned short)+sizeof(int),0);

/*  
  if(send(socketID,&m->length,sizeof(unsigned short),0)!=sizeof(unsigned short)) {
    return; // error while sending -> ignore
  }
  if(send(socketID,&m->messageID,sizeof(int),0)!=sizeof(int)) {
    return; // error while sending -> ignore
  }
  if(send(socketID,m->buffer,m->length,0)!=m->length) {
    return; // error while sending -> ignore
  } */
}

Message *Connection::getMessage()
{
  fd_set readSet;
  timeval tv;
  Message *m;

  FD_ZERO(&readSet);
  FD_SET(socketID,&readSet);

  tv.tv_sec=0;
  tv.tv_usec=0;

  if(select(socketID+1,&readSet,NULL,NULL,&tv)) {
    m=new Message;
    if(recv(socketID,&m->length,sizeof(unsigned short),MSG_WAITALL)!=sizeof(unsigned short)) {
      delete m;
      std::cerr << "Connection::getMessage(): unable to read packet length" << std::endl;
      return 0; // error while recieving -> ignore
    }
    if(recv(socketID,&m->messageID,sizeof(int),MSG_WAITALL)!=sizeof(int)) {
      delete m;
      std::cerr << "Connection::getMessage(): unable to read packet ID" << std::endl;
      return 0; // error while recieving -> ignore
    }
    m->buffer=new char[m->length];
    if(recv(socketID,m->buffer,m->length,MSG_WAITALL)!=m->length) {
      delete m;
      std::cerr << "Connection::getMessage(): unable to read packet data" << std::endl;
      return 0; // error while recieving -> ignore
    }
    //    std::cout << "Connection::getMessage(): m->length: " << m->length << std::endl;
    //    std::cout << "Connection::getMessage(): m->messageID: " << m->messageID << std::endl;
/*    readData((char*)&m->length,sizeof(unsigned short));
    std::cout << "Connection::getMessage(): m->length: " << m->length << std::endl;
    readData((char*)&m->messageID,sizeof(int));
    std::cout << "Connection::getMessage(): m->messageID: " << m->messageID << std::endl;
    readData(m->buffer,m->length); */
    return m;
  }
     
  return 0;
}

void Connection::readData(char *buffer, unsigned int length)
{
  unsigned int readLength;
  
  readLength=0;
  
  do {
    int thisLength=recv(socketID,buffer+readLength,length-readLength,MSG_WAITALL);
    
    if(thisLength>0)
      readLength+=thisLength;
    
    std::cout << "Connection::readData: readLength=" << readLength << std::endl;
  } while(readLength<length);
}
