#include "message.h"

Message::Message()
{
  buffer=0;
  length=0;
  messageID=0;
}

Message::~Message()
{
  delete buffer;
}

MessageReader::MessageReader(Message *m)
{
  message=m;

  pointer=message->buffer;
}

int MessageReader::readByte()
{
  char data;

  if(pointer+(sizeof(char))>message->buffer+message->length) {
    // TODO: throw error
  }

  data=*(char *)pointer;
  pointer+=sizeof(char);

  return data;
}

short MessageReader::readShort()
{
  short data;

  if(pointer+(sizeof(short))>message->buffer+message->length) {
    // TODO: throw error
  }

  data=*(short *)pointer;
  pointer+=sizeof(short);

  return data;
}

int MessageReader::readInt()
{
  char data;

  if(pointer+(sizeof(int))>message->buffer+message->length) {
    // TODO: throw error
  }

  data=*(int *)pointer;
  pointer+=sizeof(int);

  return data;
}

float MessageReader::readFloat()
{
  float data;

  if(pointer+(sizeof(float))>message->buffer+message->length) {
    // TODO: throw error
  }

  data=*(float *)pointer;
  pointer+=sizeof(float);

  return data;
}

std::string MessageReader::readString()
{
  std::string data;
  int length;

  length=readShort();
  for(int i=0;i<length;i++) {
    data+=readByte();
  }

  return data;
}

MessageWriter::MessageWriter(Message *m)
{
  message=m;

  if(m->buffer)
    delete m->buffer;

  m->length=0;
  pointer=0;
}

void MessageWriter::writeByte(int data)
{
  if(pointer+sizeof(char)>message->buffer+message->length) {
    resizeBuffer(message->length+sizeof(char));
  }

  *(char *)pointer=data;
  pointer+=sizeof(char);
}

void MessageWriter::writeShort(short data)
{
  if(pointer+sizeof(short)>message->buffer+message->length) {
    resizeBuffer(message->length + sizeof(short) - (message->buffer + message->length - pointer));
  }

  *(short *)pointer=data;
  pointer+=sizeof(short);
}

void MessageWriter::writeInt(int data)
{
  if(pointer+sizeof(int)>message->buffer+message->length) {
    resizeBuffer(message->length + sizeof(int) - (message->buffer + message->length - pointer));
  }

  *(int *)pointer=data;
  pointer+=sizeof(int);
}

void MessageWriter::writeFloat(float data)
{
  if(pointer+sizeof(float)>message->buffer+message->length) {
    resizeBuffer(message->length + sizeof(float) - (message->buffer + message->length - pointer));
  }

  *(float *)pointer=data;
  pointer+=sizeof(float);
}

void MessageWriter::writeString(std::string data)
{
  if(data.length()>32767) {
    // TODO: throw error here
  }
  
  writeShort(data.length());
  for(int i=0;i<data.length();i++) {
    writeByte(data[i]);
  }
}

void MessageWriter::resizeBuffer(int newLength)
{
  char *origBuffer=message->buffer;
  int offset;

  offset=pointer-message->buffer;
  message->buffer=new char[newLength];

  if(origBuffer!=0) {
    memcpy(message->buffer,origBuffer,message->length);
    delete origBuffer;
  }

  message->length=newLength;
  pointer=message->buffer+offset;
}
