#include "boost/python.hpp"

#include "message.h"
#include "connection.h"
#include "client.h"
#include "server.h"

using namespace boost::python;

class WrappedClient
{
 public:
  WrappedClient(PyObject *self, const Client &c);

  void processMessage(Message *m);

 private:
  PyObject *self;
  const Client &client;
};

WrappedClient::WrappedClient(PyObject *self, const Client &c)
  : client(c)
{
  this->self=self;
}

void WrappedClient::processMessage(Message *m)
{
  call_method<Message *>(self,"processMessage");
}

class WrappedServer : public Server 
{
 public:
  WrappedServer(PyObject *self);

  void processMessage(Message *m);
  void processIncomingConnection(Connection *c);

 private:
  PyObject *self;
};

WrappedServer::WrappedServer(PyObject *self)
{
  this->self=self;
}

void WrappedServer::processMessage(Message *m)
{
  call_method<Message *>(self,"processMessage");
}

void WrappedServer::processIncomingConnection(Connection *c)
{
  call_method<Connection *>(self,"processIncomingConnection");
}

BOOST_PYTHON_MODULE(net)
{
  class_<Message>("Message",init<>())
    .def("getMessageID",&Message::getMessageID)
    .def("setMessageID",&Message::setMessageID);

  class_<MessageReader>("MessageReader",init<Message *>())
    .def("resetPointer",&MessageReader::resetPointer)
    .def("endOfMessage",&MessageReader::endOfMessage)
    .def("readByte",&MessageReader::readByte)
    .def("readShort",&MessageReader::readShort)
    .def("readInt",&MessageReader::readInt)
    .def("readFloat",&MessageReader::readFloat)
    .def("readString",&MessageReader::readString);
  
  class_<MessageWriter>("MessageWriter",init<Message *>())
    .def("writeByte",&MessageWriter::writeByte)
    .def("writeShort",&MessageWriter::writeShort)
    .def("writeInt",&MessageWriter::writeInt)
    .def("writeFloat",&MessageWriter::writeFloat)
    .def("writeString",&MessageWriter::writeString);

  class_<Connection>("Connection",no_init)
    .def("sendMessage",&Connection::sendMessage)
    .def("getMessage",&Connection::getMessage,return_value_policy<reference_existing_object>());

  // TODO: client & server without protocol logic...

  class_<Client>("Client",init<>())
    .def("connectToServer",&Client::connectToServer)
    .def("step",&Client::step)
    .def("getConnection",&Client::getConnection,return_value_policy<reference_existing_object>());

  class_<Server>("Server",init<>())
    .def("createServerSocket",&Server::createServerSocket)
    .def("step",&Server::step)
    .def("broadcastMessage",&Server::broadcastMessage)
    .def("getNumConnections",&Server::getNumConnections)
    .def("getConnection",&Server::getConnection,return_value_policy<reference_existing_object>())
    .def("getIncomingConnection",&Server::getIncomingConnection,return_value_policy<reference_existing_object>());
}
