#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include <iostream>
#include <sstream>

#include "timer.h"
#include "server.h"

void Server::createServerSocket()
{
  sockaddr_in serverAddr;

  if((serverSocket=socket(PF_INET,SOCK_STREAM,0))<0) {
    // TODO: error handling goes here
    std::cerr << "Server::createServerSocket(): call to socket() failed" << std::endl;
  }

  memset((char *) &serverAddr,0,sizeof(struct sockaddr_in));
  serverAddr.sin_family=AF_INET;
  serverAddr.sin_port=htons(1234);
  serverAddr.sin_addr.s_addr=INADDR_ANY;

  if(bind(serverSocket,(sockaddr*)&serverAddr,sizeof(sockaddr_in))!=0) {
    // TODO: error handling goes here
    std::cerr << "Server::createServerSocket(): call to bind() failed" << std::endl;
    switch(errno) {
    case EBADF:
      std::cerr << "Server::createServerSocket(): reason: bad file descriptor" << std::endl;
      break;
    case EINVAL:
      std::cerr << "Server::createServerSocket(): reason: socket already bound to an address" << std::endl;
      break;
    case EACCES:
      std::cerr << "Server::createServerSocket(): reason: not sufficient rights" << std::endl;
      break;
    case ENOTSOCK:
      std::cerr << "Server::createServerSocket(): reason: not a socket file descriptor" << std::endl;
      break;
    default:
      std::cerr << "Server::createServerSocket(): reason is unknown, errno=" << errno << std::endl;
      break;
    }
  }

  if(listen(serverSocket,10)==-1) {
    // TODO: error handling goes here
    std::cerr << "Server::createServerSocket(): call to listen() failed" << std::endl;
  }

  // set server socket non-blocking
  if(fcntl(serverSocket,F_SETFL,O_NONBLOCK)!=0) {
    // TODO: error handling goes here
    std::cerr << "Server::createServerSocket(): call to fcntl() failed" << std::endl;
  }
}

void Server::processIncomingConnection(Connection *c)
{
  std::cout << "Server::processIncomingConnection()" << std::endl;
}

void Server::processIncomingConnections()
{
  fd_set readSet;
  timeval tv;
  sockaddr_in incomingAddr;
  Connection *conn;

  FD_ZERO(&readSet);
  FD_SET(serverSocket,&readSet);
  tv.tv_sec=0;
  tv.tv_usec=0;

  if(select(serverSocket+1,&readSet,NULL,NULL,&tv)) {
    int newSocket;

    std::cerr << "Server::processIncomingConnections(): getting a connection" << std::endl;

    //    newSocket=accept(serverSocket,(sockaddr*)&incomingAddr,sizeof(incomingAddr));
    newSocket=accept(serverSocket,0,0);
    if(newSocket<0) {
      // TODO: error handling goes here
    }
    
    // create a new Connection object on the socket
    conn=new Connection(newSocket);

    connections.push_back(conn);
    processIncomingConnection(conn);
  }
}

Connection *Server::getIncomingConnection()
{
  fd_set readSet;
  timeval tv;
  sockaddr_in incomingAddr;
  Connection *conn;

  FD_ZERO(&readSet);
  FD_SET(serverSocket,&readSet);
  tv.tv_sec=0;
  tv.tv_usec=0;

  if(select(serverSocket+1,&readSet,NULL,NULL,&tv)) {
    int newSocket;

    std::cerr << "Server::getIncomingConnection(): getting a connection" << std::endl;

    //    newSocket=accept(serverSocket,(sockaddr*)&incomingAddr,sizeof(incomingAddr));
    newSocket=accept(serverSocket,0,0);
    if(newSocket<0) {
      // TODO: error handling goes here
    }
    
    // create a new Connection object on the socket
    conn=new Connection(newSocket);

    connections.push_back(conn);
    return conn;
  }

  return 0;
}

void Server::processMessage(Message *m)
{
  std::cout << "Server::processMessage(): messageID=" << m->getMessageID() << std::endl;
}

void Server::step()
{
  Message *m;

  processIncomingConnections();

  for(int i=0;i<connections.size();i++) {
    while(m=connections[i]->getMessage()) {
      processMessage(m);
    }
  }
}

void Server::broadcastMessage(Message *m)
{
  // send message to all clients
  for(int i=0;i<connections.size();i++) {
    connections[i]->sendMessage(m);
  }
}

int Server::getNumConnections()
{
  return connections.size();
}

Connection *Server::getConnection(int numConnection)
{
  return connections[numConnection];
}
