import net
from soldier import *
from unit import *

class Player:
    def __init__(self, world, connection, name, teamID, redTeam, blueTeam):
        self.world=world
        self.connection=connection
        self.name=name
        self.teamID=teamID
        self.redTeam=redTeam
        self.blueTeam=blueTeam
        # create player's squad:
        self.units=[]
        for i in range(10):
            if teamID==1:
                teamColor=(255,0,0)
            else:
                teamColor=(0,0,255)

            soldier=Soldier(self.world,teamColor)
            self.units.append(soldier)
            world.registerUnit(soldier)

    def arrangeUnits(self, startpos):
        startposx, startposy=startpos
        offset=0
        for unit in self.units:
            unit.setPosition(startposx,startposy+offset)
            offset+=17

    def getConnection(self):
        return self.connection

    def getName(self):
        return self.name

    def getTeamID(self):
        return self.teamID

    def handleInput(self):
        clientPacket=self.connection.getMessage()
        while clientPacket <> None:
            clientPacketReader=net.MessageReader(clientPacket)
            messageID=clientPacket.getMessageID()
            # parse and handle packet
            if messageID==1:
                # move a unit
                clientPacketReader.readString()
                unitID=clientPacketReader.readByte()
                movex=clientPacketReader.readShort()
                movey=clientPacketReader.readShort()
                self.units[unitID].setMoveTarget(movex,movey)
            elif messageID==2:
                # attack an enemy unit
                clientPacketReader.readString()
                unitID=clientPacketReader.readByte()
                teamID=clientPacketReader.readByte()
                enemyName=clientPacketReader.readString()
                enemyID=clientPacketReader.readByte()
                # find target unit
                if teamID==1:
                    for i in self.redTeam.getPlayers():
                        if i.getName()==enemyName:
                            print "attacking unit"
                            self.units[unitID].setAttackTarget(i.getUnits()[enemyID])
                else:
                    for i in self.blueTeam.getPlayers():
                        if i.getName()==enemyName:
                            print "attacking unit"
                            self.units[unitID].setAttackTarget(i.getUnits()[enemyID])                    

            clientPacket=self.connection.getMessage()

    def serializeStatus(self):
        status=net.Message()
        statusWriter=net.MessageWriter(status)
        statusWriter.writeString(self.name)
        statusWriter.writeByte(self.teamID)
        statusWriter.writeByte(len(self.units))
        
        for unit in self.units:
            unit.serializeStatus(statusWriter)
            
        return status

    def unserializeStatus(self, messageReader):
        messageReader.resetPointer()
        name=messageReader.readString()
        if name!=self.name:
            return

        teamID=messageReader.readByte()
        if teamID!=self.teamID:
            return

        unitCount=messageReader.readByte()
        for i in range(unitCount):
            self.units[i].unserializeStatus(messageReader)

    def getHealth(self):
        totalHealth=0
        for unit in self.units:
            totalHealth=totalHealth+unit.getCurrentHealth()

        return totalHealth

    def getMaxHealth(self):
        totalHealth=0
        for unit in self.units:
            totalHealth=totalHealth+unit.getMaxHealth()

        return totalHealth

    def ownsUnit(self, unit):
        return (unit in self.units)

    def getUnitID(self, unit):
        return self.units.index(unit)

    def getUnits(self):
        return self.units
