import sys, pygame
import net
import loadmap, inputstring, inputside, world, viewport, minimap, team, player

playerName=""
teamID=0
serverIP=""
myPlayer=None
firstPositionUpdate=True

def initialInput():
    global playerName, teamID, serverIP
    playerName=inputstring.inputString(screen,"Enter player name:")
    teamID=inputside.inputSide(screen)
    serverIP=inputstring.inputString(screen,"Enter server IP:").encode("ascii")

def connectToServer():
    global playerName, teamID, serverIP, client, world
    client.connectToServer(serverIP,1234)

    # wait for greeting
    greeting=client.getConnection().getMessage()
    while greeting==None:
        greeting=client.getConnection().getMessage()
        
    reader=net.MessageReader(greeting)        
    text=reader.readString()
    print text
        
    worldWidth=reader.readShort()
    worldHeight=reader.readShort()

    # send player info

    playerInfo=net.Message()
    playerInfoWriter=net.MessageWriter(playerInfo)
    playerInfoWriter.writeString(playerName.encode("ascii"))
    playerInfoWriter.writeByte(teamID)

    client.getConnection().sendMessage(playerInfo)

    print "Sent player info, waiting for map"

    # read map
    print worldWidth, worldHeight

    numLines=0
    map=[]
    while numLines < worldWidth:
        percentage=(1.0*numLines)/worldWidth
        loadmap.loadMap(screen, percentage)

        line=client.getConnection().getMessage()
        if line <> None:
            numLines=numLines+1
            lineData=[]
            reader=net.MessageReader(line)
            for i in range(worldHeight):
                tile=reader.readByte()
                lineData.append(tile)
            #print tile
            map.append(lineData)

    world=world.World()
    world.setMap(worldWidth,worldHeight,map)

def waitForStart():
    global redTeam, blueTeam, myPlayer, playerName, viewport
    
    gameStarted=False
    font=pygame.font.Font("trebucbd.ttf",20)
    pleaseWait=font.render("Please wait for game to begin...",True,(255,255,255),(0,0,0))
    while not gameStarted:
        packet=client.getConnection().getMessage()
        if packet <> None:
            packetReader=net.MessageReader(packet)
            status=packetReader.readByte()
            if status==1:
                gameStarted=True
                numRedPlayers=packetReader.readByte()
                for i in range(numRedPlayers):
                    thisPlayerName=packetReader.readString()
                    redTeam.addPlayer(player.Player(world,None,thisPlayerName,1,redTeam,blueTeam))
                    if playerName == thisPlayerName:
                        myPlayer=redTeam.getPlayers()[-1]
                numBluePlayers=packetReader.readByte()
                for i in range(numBluePlayers):
                    thisPlayerName=packetReader.readString()
                    blueTeam.addPlayer(player.Player(world,None,thisPlayerName,2,redTeam,blueTeam))
                    if playerName == thisPlayerName:
                        myPlayer=blueTeam.getPlayers()[-1]
                
        # draw some waiting screen here
        screen.fill((0,0,0))
        screen.blit(pleaseWait,pleaseWait.get_rect().move((400-pleaseWait.get_width()/2,300-pleaseWait.get_height()/2)))
        pygame.display.flip()

def handleServerMessages():
    global client, redTeam, blueTeam, firstPositionUpdate, viewport, myPlayer, gameResult
    message=client.getConnection().getMessage()
    while message <> None:
        messageReader=net.MessageReader(message)
        if message.getMessageID()==1:
            # game state change
            status=messageReader.readByte()
            if status==2: # game ended
                print "game ended"
                gameResult=messageReader.readByte()
                return False
        elif message.getMessageID()==2:
            # situation update
            for player in redTeam.getPlayers():
                player.unserializeStatus(messageReader)
            for player in blueTeam.getPlayers():
                player.unserializeStatus(messageReader)
            if firstPositionUpdate: # center viewport on player position
                viewportX, viewportY =myPlayer.getUnits()[1].getPosition()
                viewportX=viewportX-400
                viewportY=viewportY-300
                # NOTE: use viewport.moveWorldPosition because setWorldPosition seems to be buggy
                # if viewportX < 0:
                #     viewportX=0
                # if viewportY < 0:
                #     viewportY=0
                # viewport.setWorldPosition(viewportX,viewportY)
                viewport.setWorldPosition(viewportX,viewportY)
                firstPositionUpdate=False
                

        message=client.getConnection().getMessage()

    return True

frameCount=0

def moveUnit(unit, movex, movey):
    global client, myPlayer
    
    moveMessage=net.Message()
    moveMessage.setMessageID(1)
    moveMessageWriter=net.MessageWriter(moveMessage)
    moveMessageWriter.writeString(myPlayer.getName().encode("ascii"))
    moveMessageWriter.writeByte(myPlayer.getUnitID(unit))
    moveMessageWriter.writeShort(movex)
    moveMessageWriter.writeShort(movey)

    client.getConnection().sendMessage(moveMessage)

def attackUnit(unit, enemy):
    global client, myPlayer, redTeam, blueTeam

    attackMessage=net.Message()
    attackMessage.setMessageID(2)
    attackMessageWriter=net.MessageWriter(attackMessage)
    attackMessageWriter.writeString(myPlayer.getName().encode("ascii"))
    attackMessageWriter.writeByte(myPlayer.getUnitID(unit))

    if myPlayer.getTeamID() == 2:
        for i in redTeam.getPlayers():
            if i.ownsUnit(enemy):
                attackMessageWriter.writeByte(i.getTeamID())
                attackMessageWriter.writeString(i.getName().encode("ascii"))
                attackMessageWriter.writeByte(i.getUnitID(enemy))
    else:
        for i in blueTeam.getPlayers():
            if i.ownsUnit(enemy):
                attackMessageWriter.writeByte(i.getTeamID())
                attackMessageWriter.writeString(i.getName().encode("ascii"))
                attackMessageWriter.writeByte(i.getUnitID(enemy))


    client.getConnection().sendMessage(attackMessage)

def clientLoop():
    global viewport, frameCount, myPlayer
    
    while 1:
        frameCount=frameCount+1
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return
                if event.key == pygame.K_c:
                    for i in world.units:
                        i.setSelected(0)
            elif event.type == pygame.MOUSEBUTTONDOWN:
                mousex, mousey = event.pos
                worldx, worldy = viewport.getWorldCoordinates(mousex, mousey)
                if event.button == 3:
                    unit=world.unitAtPosition(worldx, worldy)
                    if unit!=0:
                        # check if the target unit is owned by opponents
                        # if so, attack
                        if myPlayer.getTeamID()==2:
                            for i in redTeam.getPlayers():
                                if i.ownsUnit(unit):
                                    for j in world.units:
                                        if j.getSelected() == 1:
                                            if myPlayer.ownsUnit(j):
                                                attackUnit(j,unit)
                        else:
                            for i in blueTeam.getPlayers():
                                if i.ownsUnit(unit):
                                    for j in world.units:
                                        if j.getSelected() == 1:
                                            if myPlayer.ownsUnit(j):
                                                attackUnit(j,unit)
                    else:
                        for i in world.units:
                            if i.getSelected() == 1:
                                if myPlayer.ownsUnit(i):
                                    # tell server to move this unit
                                    moveUnit(i,worldx,worldy)
                elif event.button == 1:
                    unit=world.unitAtPosition(worldx, worldy)
                    if unit != 0:
                        unit.setSelected(1-unit.getSelected())

        mposx, mposy = pygame.mouse.get_pos()
        if mposx > viewport.screenrect.left and mposx < viewport.screenrect.left+10:
            viewport.moveWorldPosition(-3,0)
        elif mposx > viewport.screenrect.right-10 and mposx < viewport.screenrect.right:
            viewport.moveWorldPosition(3,0)

        if mposy > viewport.screenrect.top and mposy < viewport.screenrect.top+10:
            viewport.moveWorldPosition(0,-3)
        elif mposy > viewport.screenrect.bottom-10 and mposy < viewport.screenrect.bottom:
            viewport.moveWorldPosition(0,3)

        if not handleServerMessages():
            return

        viewport.draw()
        minimap.drawMap()
        pygame.display.flip()    

def postGameLoop():
    global viewport, gameResult
    
    messageFont=pygame.font.Font("trebucbd.ttf",30)

    while 1:
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                return
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    return

        mposx, mposy = pygame.mouse.get_pos()
        if mposx > viewport.screenrect.left and mposx < viewport.screenrect.left+10:
            viewport.moveWorldPosition(-10,0)
        elif mposx > viewport.screenrect.right-10 and mposx < viewport.screenrect.right:
            viewport.moveWorldPosition(10,0)

        if mposy > viewport.screenrect.top and mposy < viewport.screenrect.top+10:
            viewport.moveWorldPosition(0,-10)
        elif mposy > viewport.screenrect.bottom-10 and mposy < viewport.screenrect.bottom:
            viewport.moveWorldPosition(0,10)

        viewport.draw()
        minimap.drawMap()

        if gameResult==0:
            message="Game is a draw"
            messageColor=(255,255,255)
        elif gameResult==1:
            message="Red team prevails"
            messageColor=(255,0,0)
        elif gameResult==2:
            message="Blue team prevails"
            messageColor=(0,0,255)
        elif gameResult==3:
            message="Game aborted"
            messageColor=(255,255,255)

        messageBitmap=messageFont.render(message,True,messageColor,(0,0,0))
        screen.blit(messageBitmap,messageBitmap.get_rect().move((400-messageBitmap.get_width()/2,75-messageBitmap.get_height()/2)))


        pygame.display.flip()        

pygame.init()

screen=pygame.display.set_mode((800,600),pygame.FULLSCREEN)

initialInput()

client=net.Client()
connectToServer()

viewport=viewport.Viewport(screen,world)
viewport.setScreenPosition(0,0,screen.get_rect().width,screen.get_rect().height)
viewport.setWorldPosition(0,0)
viewport.loadTiles()

minimap=minimap.MiniMap(screen,world,viewport)
minimap.prerenderMap()

redTeam=team.Team()
blueTeam=team.Team()

waitForStart()

print myPlayer.getName()

startTime=pygame.time.get_ticks()
clientLoop()
endTime=pygame.time.get_ticks()

print "frame count: ", frameCount
print "frames per second: ", frameCount*1000.0/(endTime-startTime)

# print result on top of scrollable map
postGameLoop()
