import sys, pygame
import net
import world, soldier, team
from player import *

gameResult=0
gameOver=False

def drawStats():
    global redTeam, blueTeam, playerFont, headingFont, gameResult, gameOver

    screen.fill((0,0,0))

    heading=headingFont.render("SQUAD",True,(255,255,255),(0,0,0))
    screen.blit(heading,heading.get_rect().move((400-heading.get_width()/2,40-heading.get_height()/2)))

    # red team stats:
    posY=100
    for player in redTeam.getPlayers():
        playerName=playerFont.render(player.getName(),True,(255,0,0),(0,0,0))
        screen.blit(playerName,playerName.get_rect().move((50,posY)))

        width=200*player.getHealth()/player.getMaxHealth()
        screen.fill((255,0,0),(50,posY+20,width,5))

        posY=posY+30
        
    # blue team stats:
    posY=100
    for player in blueTeam.getPlayers():
        playerName=playerFont.render(player.getName(),True,(0,0,255),(0,0,0))
        screen.blit(playerName,playerName.get_rect().move((750-playerName.get_width(),posY)))

        width=200*player.getHealth()/player.getMaxHealth()
        screen.fill((0,0,255),(750-width,posY+20,width,5))

        posY=posY+30

    if gameOver:
        if gameResult==0:
            message="Game is a draw"
            messageColor=(255,255,255)
        elif gameResult==1:
            message="Red team prevails"
            messageColor=(255,0,0)
        elif gameResult==2:
            message="Blue team prevails"
            messageColor=(0,0,255)
        elif gameResult==3:
            message="Game aborted"
            messageColor=(255,255,255)

        messageBitmap=headingFont.render(message,True,messageColor,(0,0,0))
        screen.blit(messageBitmap,messageBitmap.get_rect().move((400-messageBitmap.get_width()/2,75-messageBitmap.get_height()/2)))

    pygame.display.flip()

def handleIncomingConnection(connection):
    global world, redTeam, blueTeam

    # send greeting
    greeting=net.Message()
    writer=net.MessageWriter(greeting)
    writer.writeString("Hello, SquadClient")
    writer.writeShort(world.getWidth())
    writer.writeShort(world.getHeight())
    connection.sendMessage(greeting)

    print "handleIncomingConnection: waiting for player info"

    # wait for player info
    playerInfo=connection.getMessage()
    while playerInfo==None:
        playerInfo=connection.getMessage()

    print "handleIncomingConnection: parsing player info"

    playerInfoReader=net.MessageReader(playerInfo)
    playerName=playerInfoReader.readString()
    print "playerName: " + playerName
    playerTeamID=playerInfoReader.readByte()
    print "playerTeamID: " + str(playerTeamID)

    player=Player(world,connection,playerName,playerTeamID,redTeam,blueTeam)
    if playerTeamID==1:
        redTeam.addPlayer(player)
        player.arrangeUnits((160,200*len(redTeam.getPlayers())))
    elif playerTeamID==2:
        blueTeam.addPlayer(player)
        player.arrangeUnits((world.getWidth()*16-160,200*len(blueTeam.getPlayers())))


    print "handleIncomingConnection: sending map"

    # send map    
    for i in range(world.getWidth()):
        mapLine=net.Message()
        mapWriter=net.MessageWriter(mapLine)
        for j in range(world.getHeight()):
            mapWriter.writeByte(world.getTile(i,j))
        connection.sendMessage(mapLine)

def preGameLoop():
    global server, redTeam, blueTeam
    
    server.createServerSocket()

    while 1:
        for event in pygame.event.get():
            if event.type==pygame.QUIT:
                # TODO: notify clients of server termination
                sys.exit()
            elif event.type==pygame.KEYDOWN:
                if event.key==pygame.K_ESCAPE:
                    # TODO: see above
                    sys.exit()
                elif event.key==pygame.K_RETURN:
                    return #starts game

        # handle new incoming connections:
        newConnection=server.getIncomingConnection()
        if newConnection <> None:
            # handle connection
            handleIncomingConnection(newConnection)

        drawStats()
        pygame.time.wait(20)

def gatherClientInput():
    global redTeam, blueTeam
    
    for player in redTeam.getPlayers():
        player.handleInput()

    for player in blueTeam.getPlayers():
        player.handleInput()

def simulateStep():
    global redTeam, blueTeam

    # simulate units
    world.updateUnits()

    # terminate game if one team is wiped out
    if redTeam.getHealth()==0 or blueTeam.getHealth()==0:
        return True

def updateClients():
    global redTeam, blueTeam, server
    # use server.broadcastMessage() to send status updates

    for player in redTeam.getPlayers():
        status=player.serializeStatus()
        status.setMessageID(2) # situation update
        server.broadcastMessage(status)

    for player in blueTeam.getPlayers():
        status=player.serializeStatus()
        status.setMessageID(2) # situation update
        server.broadcastMessage(status)

    # TODO: wait for client acks

def gameLoop():
    global server,gameResult,gameOver
    
    # send game start notification to clients:
    gameStart=net.Message()
    gameStart.setMessageID(1) # game state change
    gameStartWriter=net.MessageWriter(gameStart)
    gameStartWriter.writeByte(1) # status 1 means game starts now
    #include player list:
    gameStartWriter.writeByte(len(redTeam.getPlayers()))
    for player in redTeam.getPlayers():
        gameStartWriter.writeString(player.getName().encode("ascii"))
    gameStartWriter.writeByte(len(blueTeam.getPlayers()))
    for player in blueTeam.getPlayers():
        gameStartWriter.writeString(player.getName().encode("ascii"))
    server.broadcastMessage(gameStart)

    # main game loop:
    gameOver=False
    while not gameOver:
        for event in pygame.event.get():
            if event.type==pygame.QUIT:
                # TODO: notify clients of server termination
                gameOver=True
            elif event.type==pygame.KEYDOWN:
                if event.key==pygame.K_ESCAPE:
                    # TODO: see above
                    gameOver=True

        pygame.time.wait(20)
        gatherClientInput()
        gameOver=simulateStep()
        updateClients()
        drawStats()

        

    # send game end notification to clients:
    gameEnd=net.Message()
    gameEnd.setMessageID(1) # game state change
    gameEndWriter=net.MessageWriter(gameEnd)
    gameEndWriter.writeByte(2) # status 2 means game ends now
    if redTeam.getHealth()==0 and blueTeam.getHealth()==0:
        gameResult=0 # game is a draw
    elif redTeam.getHealth()==0:
        gameResult=2 # blue team prevails
    elif blueTeam.getHealth()==0:
        gameResult=1 # red team prevails
    else:
        gameResult=3 # game terminated

    gameEndWriter.writeByte(gameResult)
    server.broadcastMessage(gameEnd)

def postGameLoop():
    while 1:
        for event in pygame.event.get():
            if event.type==pygame.QUIT:
                return
            elif event.type==pygame.KEYDOWN:
                return

        drawStats()

pygame.init()
screen=pygame.display.set_mode((800,600),pygame.FULLSCREEN)

playerFont=pygame.font.Font("trebucbd.ttf",15)
headingFont=pygame.font.Font("trebucbd.ttf",30)

world=world.World()
world.loadMap("Test")

redTeam=team.Team()
blueTeam=team.Team()

server=net.Server()

preGameLoop()
gameLoop()
postGameLoop()
