import math, pygame

class Unit:
    def __init__(self,world):
        self.world=world
        self.position=(0,0)
        self.movetarget=(0,0)
        self.attacktarget=None
        self.selected=0
        self.isshooting=False

    def createFrame(self):
        self.frame=pygame.Surface(self.size)
        self.frame.fill((255,0,255))
        self.frame.set_colorkey((255,0,255))
        sizex, sizey = self.size
        self.frame.lock()
        for i in range(sizex):
            self.frame.set_at((i,0),(255,255,255))
            self.frame.set_at((i,sizey-1),(255,255,255))
        for i in range(sizey):
            self.frame.set_at((0,i),(255,255,255))
            self.frame.set_at((sizex-1,i),(255,255,255))
        self.frame.unlock()

    def updateFrame(self):
        sizex, sizey = self.size
        for i in range(sizex):
            self.frame.set_at((i,1),(0,0,0))

        healthbarend=int(sizex*((1.0*self.getCurrentHealth())/self.getMaxHealth()))
        for i in range(healthbarend):
            self.frame.set_at((i,1),(255,0,0))

    def getFrame(self):
        return self.frame

    def setMovementSpeed(self, speed):
        self.movementspeed=speed

    def getMovementSpeed(self):
        return movementspeed

    def setMaxHealth(self, maxhealth):
        self.maxhealth=maxhealth

    def getMaxHealth(self):
        return self.maxhealth

    def setCurrentHealth(self,health):
        self.currenthealth=health

    def getCurrentHealth(self):
        return self.currenthealth

    def setMaxDamage(self,damage):
        self.maxdamage=damage

    def getMaxDamage(self):
        return self.maxdamage

    def setShootingRange(self,shootingrange):
        self.shootingrange=shootingrange

    def getShootingRange(self):
        return self.shootingrange

    def inflictDamage(self,damage):
        self.currenthealth=self.currenthealth-damage
        if self.currenthealth < 0:
            self.currenthealth=0

    def getSelected(self):
        return self.selected

    def setSelected(self, selected):
        self.selected=selected

    def setSize(self, sizex, sizey):
        self.size=(sizex, sizey)

    def getSize(self):
        return self.size

    def getPosition(self):
        return self.position

    def setPosition(self, x, y):
        self.position=(x,y)
        self.movetarget=(x,y)

    def move(self, deltax, deltay):
        x, y=self.position
        self.position=(x+deltax,y+deltay)

    def setMoveTarget(self,movex,movey):
        self.attacktarget=None
        self.movetarget=movex, movey

    def setAttackTarget(self,unit):
        self.attacktarget=unit

    def isShooting(self):
        return self.isshooting

    def update(self):
        if self.currenthealth<=0:
            return
        
        self.isshooting=False
        if self.attacktarget <> None:
            self.movetarget = self.attacktarget.getPosition()
            targetposx, targetposy = self.movetarget
            posx, posy = self.position
            deltax = targetposx-posx
            deltay = targetposy-posy

            # if distance is short enough, this unit can shoot its target
            distance=math.sqrt(deltax*deltax+deltay*deltay)
            if distance < self.shootingrange:
                self.attacktarget.inflictDamage(self.maxdamage *  (1-(distance/self.shootingrange)))
                self.isshooting=True

                if self.attacktarget.getCurrentHealth()==0:
                    self.attacktarget=None
        
        if self.position != self.movetarget:
            posx, posy = self.position
            movex, movey = self.movetarget
            deltax=movex-posx
            deltay=movey-posy
            distance=math.sqrt(deltax*deltax+deltay*deltay)
            # if the distance is too small the unit might overshoot the target position,
            # so this is handled seperately by immediately moving the unit to the target
            if distance > self.movementspeed:
                directionx=deltax/distance
                directiony=deltay/distance
                posx=posx+self.movementspeed*directionx
                posy=posy+self.movementspeed*directiony
            else:
                posx=movex
                posy=movey

            # check for collisions with other units in new position and don't move
            # if there are any (world.getCollidingObjects always returns this unit!)
            sizex, sizey = self.size
            numcolliders=len(self.world.getCollidingObjects((posx,posy,sizex,sizey)))
            if numcolliders > 1:
                posx, posy = self.position

            self.position=(posx, posy)

    def rotateBitmap(self, bitmap):
        angle=0
        if self.position != self.movetarget:
            posx, posy = self.position
            movex, movey = self.movetarget
            deltax=movex-posx
            deltay=movey-posy
            angle=180*math.atan2(deltax,deltay)/math.pi                

#        rotated=pygame.transform.rotozoom(self.idle,angle,1)
        rotated=pygame.transform.rotate(bitmap,angle)
        # unfortunately, rotozoom displaces the image; try to correct it:
        finished=pygame.Surface(self.size)
        finished.set_colorkey((255,0,255))
        finished.fill((255,0,255))
        frame=self.idle.get_rect()
        frame=frame.move(-(rotated.get_rect().width-frame.width)/2,-(rotated.get_rect().height-frame.height)/2)
        finished.blit(rotated,frame)

        return finished

    def shadeBitmap(self, bitmap, color):
        colorr, colorg, colorb = color
        pixels=pygame.surfarray.array3d(bitmap)
        for i in range(bitmap.get_rect().width):
            for j in range(bitmap.get_rect().height):
                if pixels[i][j][0] == 0 and pixels[i][j][2] == 0:
                    pixels[i][j][0]=colorr * pixels[i][j][1]/255
                    pixels[i][j][1]=colorg * pixels[i][j][1]/255
                    pixels[i][j][2]=colorb * pixels[i][j][1]/255

        pygame.surfarray.blit_array(bitmap,pixels)
        return bitmap

    def serializeStatus(self, messageWriter):
        posx, posy = self.position
        messageWriter.writeShort(int(round(posx)))
        messageWriter.writeShort(int(round(posy)))
        movex, movey = self.movetarget
        messageWriter.writeShort(int(round(movex)))
        messageWriter.writeShort(int(round(movey)))
        messageWriter.writeByte(int(round(self.currenthealth)))
        messageWriter.writeByte(self.isshooting)

    def unserializeStatus(self, messageReader):
        posx=messageReader.readShort()
        posy=messageReader.readShort()
        self.position=(posx,posy)
        movex=messageReader.readShort()
        movey=messageReader.readShort()
        self.movetarget=(movex,movey)
        newhealth=messageReader.readByte()
        if newhealth <> self.currenthealth:
            self.currenthealth=newhealth
            self.updateFrame()
        self.isshooting=messageReader.readByte()
