import pygame

class Viewport:
    def __init__(self, screen, world):
        self.screen=screen
        self.world=world

    def loadTiles(self):
        self.flat=pygame.image.load("gfx/flat.bmp")
        self.rock=pygame.image.load("gfx/rocks.bmp")
        self.water=pygame.image.load("gfx/water.bmp")

    def setScreenPosition(self, x, y, width, height):
        self.screenrect=pygame.Rect(x,y,width,height)

    def setWorldPosition(self, x, y):
        self.worldpos=(-x,-y)

    def moveWorldPosition(self, deltax, deltay):
        x, y=self.worldpos
        x=x-deltax
        y=y-deltay
        if (-x) < 0: x=0
        if (-x) > (self.world.width*16)-self.screenrect.width:
            x=-((self.world.width*16)-self.screenrect.width)
        if (-y) < 0: y=0
        if (-y) > (self.world.height*16)-self.screenrect.height:
            y=-((self.world.height*16)-self.screenrect.height)
        self.worldpos=(x, y)

    def getSize(self):
        return (self.screenrect.width, self.screenrect.height)

    def getPosition(self):
        return self.screenrect.topleft

    def getWorldPosition(self):
        return self.worldpos

    def draw(self):
        self.screen.set_clip(self.screenrect)
        self.screen.fill((0,0,0))

        # draw terrain
        for j in range(self.world.height):
            frame=self.flat.get_rect()
            frame=frame.move(self.screenrect.topleft)
            frame=frame.move(self.worldpos)
            frame=frame.move((0,16*j))
            for i in range(self.world.width):
                if self.world.map[j][i] == 1:
                    bitmap=self.flat
                elif self.world.map[j][i] == 2:
                    bitmap=self.rock
                elif self.world.map[j][i] == 3:
                    bitmap=self.water

                self.screen.blit(bitmap,frame)
                frame=frame.move((16,0))

        # draw units
        for i in self.world.units:
            unitpic=i.getBitmap()
            frame=unitpic.get_rect()
            frame=frame.move(self.screenrect.topleft)
            frame=frame.move(self.worldpos)
            frame=frame.move(i.getPosition())
            self.screen.blit(unitpic,frame)
            if i.getSelected()==1:
                framepic=i.getFrame()
                self.screen.blit(framepic,frame)

    def getWorldCoordinates(self, screenx, screeny):
        worldx, worldy=self.worldpos
        resultx=screenx-self.screenrect.left
        resulty=screeny-self.screenrect.top
        resultx=resultx-worldx
        resulty=resulty-worldy
        return (resultx, resulty)
